/**
 * 
 */
package jdbcacsess;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.xml.transform.TransformerException;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author sima
 * 
 * @param <T>
 */
public abstract class InfoListSetting<T> {

	abstract protected String getKeyValue();

	abstract protected void setKeyValue(String key);

	/**
	 * 管理対象の先頭ノードのタグ名を返却する
	 * 
	 * @return
	 */
	abstract protected String getBaseTagName();

	abstract protected T getInstance() throws JdbcacsessException;

	private SettingFile xml = SettingFile.getInstance();

	/**
	 * XML要素からインスタンスへ設定する。 但しキーは対象外であり、setKeyValueでインスタンスへ設定される
	 * 
	 * @param element
	 *            入力となるXML要素
	 */
	abstract public void setFromElement(Element element);

	/**
	 * インスタンスからXML要素へ追加設定する。 但しキーは対象外であり、getKeyValueでインスタンスから取得される。
	 * 
	 * @param element
	 *            出力先となるXML要素
	 */
	abstract public void addToElement(Element element);

	/**
	 * このクラスでパラメータを管理するtopノードを取得する。 全ての操作はこの返却されたノードを基に行う。
	 * 
	 * @return 管理対象の先頭ノード(topNode)
	 */
	protected Element topElement() {
		return xml.getRootElement(getBaseTagName() + "s");
	}

	/**
	 * 最後に処理された値をxmlプロパティからインスタンスへ読み込む。
	 * topnodeの属性curentで保持している値を最新処理キーとみなすので、currentDataSaveによって書き込まれて無い時は何もしない。
	 * また、具象クラスのキーが空文字列の場合は何もしない。
	 */
	public void currentDataLoad() {
		String key = topElement().getAttribute("current");
		if (key.equals("")) {
			return;
		}
		setKeyValue(key);
		Element element = getKeyElement(topElement(), getBaseTagName(), key);

		setFromElement(element);
		LogOutput.fine("#currentDataLoad()#" + this.toString());
	}

	/**
	 * 指定されたキー値のxmlプロパティをインスタンスへ読み込む。もし存在しなかった時は、インスタンスの変更は行わずfalseを返却する
	 * 
	 * @param key
	 * @return keyの検索結果。存在した時はtrue、存在しなかった時はfalse。
	 */
	public boolean keyDataLoad(String key) {
		Map<String, Element> map = getKeyMap(topElement(), getBaseTagName());
		Element element = map.get(key);
		if (element == null) {
			return false;
		}
		setKeyValue(key);
		setFromElement(element);
		LogOutput.fine("#keyDataLoad()#" + this.toString());
		return true;
	}

	/**
	 * 最後に処理された値をインスタンスからxmlプロパティへ書き込む 但し、キーが空文字列の場合は何もしない。
	 * @throws JdbcacsessException 
	 */
	public void currentDataSave() throws JdbcacsessException {
		if (getKeyValue().equals("")) {
			return;
		}
		topElement().setAttribute("current", getKeyValue());
		Element element = getKeyElement(topElement(), getBaseTagName(),
				getKeyValue());

		addToElement(element);
		try {
			xml.save();
		} catch (FileNotFoundException e) {
			throw new JdbcacsessException(e);
		} catch (TransformerException e) {
			throw new JdbcacsessException(e);
		} catch (IOException e) {
			throw new JdbcacsessException(e);
		}
	}

	/**
	 * 削除します。何を削除するかというと．．．．何だろう？
	 * @throws JdbcacsessException 
	 */
	public void removeParameter() throws JdbcacsessException {
		NodeList childNodes = topElement().getChildNodes();

		for (int i = 0; i < childNodes.getLength(); i++) {
			Node order = childNodes.item(i);
			if (order.getNodeType() == Node.ELEMENT_NODE) {
				if (order.getNodeName().equals(getBaseTagName())) {
					Node temp = order.getAttributes().getNamedItem("key");
					if (temp.getNodeValue().equals(getKeyValue())) {
						topElement().removeChild(order);
					}
				}
			}
		}
		setKeyValue("");
		topElement().setAttribute("current", getKeyValue());
		try {
			xml.save();
		} catch (FileNotFoundException e) {
			throw new JdbcacsessException(e);
		} catch (TransformerException e) {
			throw new JdbcacsessException(e);
		} catch (IOException e) {
			throw new JdbcacsessException(e);
		}
	}

	/**
	 * パラメータの一覧をリストで返却します
	 * 
	 * @return パラメータリスト
	 * @throws JdbcacsessException 
	 */
	public List<T> getInfoList() throws JdbcacsessException {
		List<T> list = new ArrayList<T>();
		Map<String, Element> map = getKeyMap(topElement(), getBaseTagName());
		for (Map.Entry<String, Element> item : map.entrySet()) {

			T o = getInstance();
			((InfoListSetting<?>) o).setKeyValue(item.getKey());
			((InfoListSetting<?>) o).setFromElement(item.getValue());
			list.add(o);
		}
		return list;
	}

	/**
	 * パラメータの一覧をマップで返却します
	 * 
	 * @return パラメータマップ
	 * @throws JdbcacsessException 
	 */
	public Map<String, T> getInfoMap() throws JdbcacsessException {
		Map<String, T> rtn = new HashMap<String, T>();
		Map<String, Element> map = getKeyMap(topElement(), getBaseTagName());
		for (Map.Entry<String, Element> item : map.entrySet()) {

			T o = getInstance();
			((InfoListSetting<?>) o).setKeyValue(item.getKey());
			((InfoListSetting<?>) o).setFromElement(item.getValue());
			rtn.put(item.getKey(), o);
		}
		return rtn;
	}

	private Element getKeyElement(Element element, String tagName,
			String keyValue) {
		NodeList childNodes = element.getChildNodes();
		Element resultElement = null;

		for (int i = 0; i < childNodes.getLength(); i++) {
			Node order = childNodes.item(i);
			if (order.getNodeType() == Node.ELEMENT_NODE) {
				if (order.getNodeName().equals(tagName)) {
					Node temp = order.getAttributes().getNamedItem("key");
					//
					if (temp.getNodeValue().equals(keyValue)) {
						resultElement = (Element) order;
						break;
					}
				}
			}
		}
		if (resultElement == null) {
			// 見つからなかった時は追加する
			resultElement = xml.createElement(tagName);
			resultElement.setAttribute("key", keyValue);
			element.appendChild(resultElement);
		}
		return resultElement;
	}

	private Map<String, Element> getKeyMap(Element element, String tagName) {
		NodeList childNodes = element.getChildNodes();

		Map<String, Element> map = new TreeMap<String, Element>();

		for (int i = 0; i < childNodes.getLength(); i++) {
			Node order = childNodes.item(i);
			if (order.getNodeType() == Node.ELEMENT_NODE) {
				if (order.getNodeName().equals(tagName)) {
					Node temp = order.getAttributes().getNamedItem("key");
					// 
					map.put(temp.getNodeValue(), (Element) order);
				}
			}
		}
		return map;
	}
}
