/**
 * 
 */
package jdbcacsess;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * @author sima
 * 
 */
public class SettingFile {
	private static SettingFile _instance = null;

	private final static String folderName = ".jdbcacsess";

	private Document doc;

	private Element root;

	private File file;

	/**
	 * @throws ParserConfigurationException 
	 * @throws IOException 
	 * @throws SAXException 
	 * 
	 */
	public SettingFile() throws ParserConfigurationException, SAXException, IOException {
		LogOutput.info("設定ファイルの読込開始");
			// ドキュメントビルダーファクトリを生成
			DocumentBuilderFactory dbfactory = DocumentBuilderFactory
					.newInstance();
			// ドキュメントビルダーを生成
			DocumentBuilder builder = dbfactory.newDocumentBuilder();

			String fileName = getFileName("setting.xml");
			
			file = new File(fileName);
			LogOutput.info("設定ファイルを " + file + " とします。");
			
			if (file.exists()) {
				// パースを実行してDocumentオブジェクトを取得
				doc = builder.parse(file);
			} else {
				doc = builder.getDOMImplementation().createDocument("", "root",
						null);

			}
			// ルート要素を取得（タグ名：root）
			root = doc.getDocumentElement();

		LogOutput.info("設定ファイルの読込終了");
		
		_instance = this;
	}

	/**
	 * @return
	 */
	public static SettingFile getInstance() {
		return _instance;
	}

	/**
	 * 設定ファイルディレクトリ存在チェックし、存在しない場合は新規作成します。
	 * 
	 * @return 設定ファイルディレクトリ名
	 * @throws IOException 
	 */
	public static String getUserPath(){
		// パス名の作成
		String userHome = System.getProperty("user.home");
		String userPath = userHome + System.getProperty("file.separator")
				+ folderName;

		return userPath;
	}

	public static  String getFileName(String name) {
		return getUserPath() + System.getProperty("file.separator") + name;
	}

	/**
	 * @throws TransformerException
	 * @throws IOException
	 * @throws JdbcacsessException 
	 */
	synchronized public void save() throws TransformerException, IOException, JdbcacsessException {
		LogOutput.info("設定ファイルの保存開始");
		TransformerFactory tff = TransformerFactory.newInstance();
		Transformer tf = tff.newTransformer();

		RandomAccessFile raf = new RandomAccessFile("/tmp/junk.dat", "rw");
		FileChannel channel = raf.getChannel();
		FileLock lock = channel.tryLock();
		if (lock==null){
			throw new JdbcacsessException("ロック取得失敗");
		}
		
		OutputStream os = new BufferedOutputStream(new FileOutputStream(file));
		
		// 変換
		tf.transform(new DOMSource(doc), new StreamResult(os));
		
		os.close();
		lock.release();

		LogOutput.info("設定ファイルの保存終了");
	}

	/**
	 * @param tagName
	 * @return
	 */
	synchronized public Element getRootElement(String tagName) {
		return getElement(root, tagName);
	}

	/**
	 * ノード名からノードを検索する。存在しない場合は、そのノードを作成する。 １階層しか辿らないので再帰検索は行いません。
	 * 
	 * @param node
	 *            検索対象のNode
	 * @param tagName
	 *            検索対象のタグ名（ノード名?）
	 * @return 検索結果
	 */
	synchronized public Element getElement(Element node, String tagName) {

		NodeList childNodes = node.getChildNodes();
		Element resultElement = null;

		for (int i = 0; i < childNodes.getLength(); i++) {
			Node order = childNodes.item(i);
			if (order.getNodeType() == Node.ELEMENT_NODE) {
				if (order.getNodeName().equals(tagName)) {
					resultElement = (Element) order;
					break;
				}
			}
		}
		if (resultElement == null) {
			// 見つからなかった時は追加する
			resultElement = createElement(tagName);
			node.appendChild(resultElement);
		}
		return resultElement;
	}

	/**
	 * @param tagName
	 * @return
	 */
	synchronized public Element createElement(String tagName) {
		return doc.createElement(tagName);
	}

	/**
	 * @return file を戻します。
	 */
	public File getFile() {
		return file;
	}

	/**
	 * 指定Element配下のElementの中から、指定したname要素を持つElementを返します。 複数有った場合は最初の1つだけです。
	 * 
	 * @param elementTop
	 *            検索起点の要素名
	 * @param tagName
	 *            検索起点配下の検索したい要素名
	 * @param attrName
	 *            検索したい属性の名前
	 * @param attrValue
	 *            検索したい属性の値
	 * @return
	 */
	public static Element getElementSearchingAttribute(Element elementTop,
			String tagName, String attrName, String attrValue) {
		Element guiElement = null;
		NodeList list = elementTop.getElementsByTagName(tagName);
		for (int i = 0; i < list.getLength(); i++) {
			if (list.item(i).getNodeType() == Node.ELEMENT_NODE) {
				Element element = (Element) list.item(i);
				if (element.getAttribute(attrName).equals(attrValue)) {
					guiElement = element;
					break;
				}
			}
		}
		return guiElement;
	}

	/**
	 * 指定Element配下のElementの中から、指定したname要素を持つElementを返します。 複数有った場合は最初の1つだけです。
	 * 検索属性の条件を2つ指定できます。
	 * 
	 * @param elementTop
	 *            検索起点の要素名
	 * @param tagName
	 *            検索起点配下の検索したい要素名
	 * @param attrName1
	 *            検索したい属性の名前1
	 * @param attrValue1
	 *            検索したい属性の値1
	 * @param attrName2
	 *            検索したい属性の名前2
	 * @param attrValue2
	 *            検索したい属性の値2
	 * @return
	 */
	public static Element getElementSearchingAttribute2(Element elementTop,
			String tagName, String attrName1, String attrValue1,
			String attrName2, String attrValue2) {
		Element guiElement = null;
		NodeList list = elementTop.getElementsByTagName(tagName);
		for (int i = 0; i < list.getLength(); i++) {
			if (list.item(i).getNodeType() == Node.ELEMENT_NODE) {
				Element element = (Element) list.item(i);
				if (element.getAttribute(attrName1).equals(attrValue1)
						&& element.getAttribute(attrName2).equals(attrValue2)) {
					guiElement = element;
					break;
				}
			}
		}
		return guiElement;
	}

	/**
	 * 指定した要素配下の、特定タグ名を全て要素として取得します
	 * 
	 * @param element
	 * @param tagname
	 * @return
	 */
	public static ArrayList<Element> getElements(Element element, String tagname) {
		ArrayList<Element> rtn = new ArrayList<Element>();

		NodeList list = element.getElementsByTagName(tagname);
		for (int i = 0; i < list.getLength(); i++) {
			rtn.add((Element) list.item(i));
		}
		return rtn;
	}

}
