/**
 * カラム毎の生成方法を管理します。また、生成器の呼出を行いデータ生成も行います。
 */
package jdbcacsess.createdata;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.swing.table.AbstractTableModel;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.sql.ColumnInfoResult;
import jdbcacsess.sql.ColumnInfoTable;
import jdbcacsess.sql.ConstSqlTypes;
import jdbcacsess.sql.QueryExecuteListener;
import jdbcacsess.sql.QueryExecuteStatus;
import jdbcacsess.sql.SchemaTableColumnName;
import jdbcacsess.sql.SchemaTableName;
import jdbcacsess.sql.SqlExec;

public class ColumnsTableModel extends AbstractTableModel {

	private static final long serialVersionUID = -1421584187171950474L;

	private ArrayList<ColumnInfoTable> columnInfoTables;

	private ArrayList<JPanelCreateData> jPanelCreateDatas;

	private ArrayList<QueryExecuteListener> outputListenners;

	/**
	 * 
	 */
	public ColumnsTableModel() {
		super();
		this.jPanelCreateDatas = new ArrayList<JPanelCreateData>();

		outputListenners = new ArrayList<QueryExecuteListener>();
	}

	/**
	 * 生成データの通知先を登録します。QueryExecuteListenerを実装したオブジェクトで登録して下さい。
	 * 
	 * @param listener
	 */
	public void addOutputQueryExecuteListener(QueryExecuteListener listener) {
		outputListenners.add(listener);
	}

	/**
	 * 生成データの通知先を全て削除します。
	 */
	public void removeOutputQueryExecuteListeners() {
		outputListenners.clear();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return 6;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int column) {
		switch (column) {
		case 0:
			return "生成先カラム名";
		case 1:
			return "属性";
		case 2:
			return "桁・バイト";
		case 3:
			return "小数点以下";
		case 4:
			return "生成手順";
		case 5:
			return "１件目プレビュー";
		default:
			break;
		}
		return null;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return columnInfoTables.size();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
		case 0:
			return columnInfoTables.get(rowIndex).getColumnName();
		case 1:
			return columnInfoTables.get(rowIndex).getColumnTypeName();
		case 2:
			return columnInfoTables.get(rowIndex).getColumnSize();
		case 3:
			return columnInfoTables.get(rowIndex).getDecimalDigits();
		case 4:
			return jPanelCreateDatas.get(rowIndex);
		case 5:
			JPanelCreateData jpcd = jPanelCreateDatas.get(rowIndex);
			jpcd.dataInitial();
			Object rtn = jpcd.dataNext();
			jpcd.dataFinal();
			return rtn;
		default:
			break;
		}
		return null;
	}

	/**
	 * 指定されたカラム順インデックスの生成器を返します
	 * 
	 * @param rowIndex
	 *            カラム順インデックス（画面上では行インデックス）
	 * @return 生成器
	 */
	public JPanelCreateData getJPanelCreateData(int rowIndex) {
		return (JPanelCreateData) getValueAt(rowIndex, 4);
	}

	/**
	 * 指定したカラム順インデックスの位置で、生成器を置き換えます
	 * 
	 * @param rowIndex
	 *            カラム順インデックス（画面上では行インデックス）
	 * @param aValue
	 *            置き換える生成器
	 */
	public void setJPanelCreateData(int rowIndex, JPanelCreateData aValue) {
		aValue.debugPrint();
		jPanelCreateDatas.set(rowIndex, aValue);
		fireTableRowsUpdated(rowIndex, rowIndex);
	}

	/**
	 * データを生成し、登録されている通知先に通知します。
	 * 
	 * @return
	 * @throws IOException
	 * @throws FileNotFoundException
	 */
	public void initOutput() throws IOException, FileNotFoundException {
		// 生成器の初期化
		for (JPanelCreateData jpcd : jPanelCreateDatas) {
			jpcd.dataInitial();
		}

		// 初期化通知
		for (QueryExecuteListener listener : outputListenners) {
			listener.init();
		}

		// 見出しの取得
		ArrayList<ColumnInfoResult> head = new ArrayList<ColumnInfoResult>();
		for (int columnIdx = 0; columnIdx < columnInfoTables.size(); columnIdx++) {

			if (jPanelCreateDatas.get(columnIdx) instanceof JPanelDefault) {
				// 追加対象外カラムなので何もしない
				continue;
			}
			// 保持しているColumnInfoResult => ColumnInfoTable の変換が必要になったのは設計の失敗。
			head.add(new ColumnInfoResult(columnInfoTables.get(columnIdx)));
		}

		// 見出し通知
		for (QueryExecuteListener listener : outputListenners) {
			listener.setResultHeader(head, null,false);
		}

		return;
	}

	/**
	 * データを生成し、登録されている通知先に通知します。
	 * 
	 * @param detail
	 * @throws IOException
	 */
	public void detailOutput() throws IOException {
		// 明細行分の処理

		ArrayList<Object> detail = new ArrayList<Object>();
		// 明細取得
		for (JPanelCreateData jpcd : jPanelCreateDatas) {
			if (jpcd instanceof JPanelDefault) {
				// 追加対象外カラムなので何もしない
			} else {
				detail.add(jpcd.dataNext());
			}
		}

		// 明細通知
		for (QueryExecuteListener listener : outputListenners) {
			listener.setResultDetail(detail);
		}
	}

	/**
	 * データを生成し、登録されている通知先に通知します。
	 * 
	 * @param creatCnt
	 * @throws JdbcacsessException 
	 */
	public void endOutput(int creatCnt) throws JdbcacsessException {
		// 終了通知
		QueryExecuteStatus qes = new QueryExecuteStatus(this, creatCnt, true,false);
		for (QueryExecuteListener listener : outputListenners) {
			listener.completeQuery(qes);
		}

		// 生成器の終了処理
		for (JPanelCreateData jpcd : jPanelCreateDatas) {
			jpcd.dataInitial();
		}
	}

	/**
	 * 生成器(JPanelCreateData)をマップで返却します。マップのキーはカラム名です。
	 * 
	 * @return 生成器(JPanelCreateData)のマップ
	 */
	public Map<String, JPanelCreateData> getCreateDataMap() {
		Map<String, JPanelCreateData> map = new HashMap<String, JPanelCreateData>();
		for (int i = 0; i < columnInfoTables.size(); i++) {
			map.put(columnInfoTables.get(i).getColumnName(), jPanelCreateDatas
					.get(i));
		}
		return map;
	}

	/**
	 * 生成器(JPanelCreateData)をマップで受け取り設定します。マップのキーはカラム名です。
	 * 但し、テーブル定義されているカラム名と一致した生成器のみです。
	 * 
	 * @param map
	 */
	public void setCreateDataMap(Map<String, JPanelCreateData> map) {
		for (int i = 0; i < columnInfoTables.size(); i++) {
			JPanelCreateData value = map.get(columnInfoTables.get(i)
					.getColumnName());
			if (value != null) {
				jPanelCreateDatas.set(i, value);
			}
		}
	}

	/**
	 * テーブル名を設定します。ＤＢのテーブル定義を取得し、カラム情報が決定されます。
	 * カラム情報を設定ファイルに保存せず、毎回ＤＢから取り込むのは、テーブル定義の変更に対応させる為です。
	 * 
	 * @param tableName
	 * @throws SQLException
	 */
	public void setTableName(SchemaTableName tableName) throws SQLException {

		this.columnInfoTables = SqlExec.getColumnMames(tableName);
		for (int i = 0; i < this.columnInfoTables.size(); i++) {
			ColumnInfoTable columnInfoTable = columnInfoTables.get(i);
			SchemaTableColumnName stcn = new SchemaTableColumnName(
					columnInfoTable.getSchemaTableName(), columnInfoTable
							.getColumnName());

			JPanelCreateData jpcd = null;
			if (columnInfoTable.getSqlType().getCategoly() == ConstSqlTypes.Categoly.DAYTIME) {
				jpcd = new JPanelDateTime();
			}else{
				jpcd = new JPanelSequence(stcn);
			}
			jPanelCreateDatas.add(jpcd);
		}
	}

	/**
	 * 指定したカラム順インデックスのカラム定義を返します。 セルレンダラーがツールチップを表示する時や主キーのBOLD表示等で使用されます。
	 * 
	 * @param row
	 * @return テーブル定義のカラム情報
	 */
	public ColumnInfoTable getColumnInfo(int row) {
		return columnInfoTables.get(row);
	}
}