package jdbcacsess.csv;

import java.lang.reflect.InvocationTargetException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Vector;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.PrefixStringBuffer;
import jdbcacsess.gui.JFrameSetting;
import jdbcacsess.sql.ColumnInfoResult;
import jdbcacsess.sql.ColumnInfoTable;
import jdbcacsess.sql.PrepareExecute;
import jdbcacsess.sql.PrepareExecuteStatement;
import jdbcacsess.sql.SchemaTableName;
import jdbcacsess.sql.SqlExec;
import jdbcacsess.sql.SqlMappingInfo;

/**
 * csvファイルの入力データから、更新用ＳＱＬ文を編集し実行します。<br>
 * 以下３種類のＳＱＬ文を編集出来ます。<br>
 * １．更新カラム名付きのINSERT文<br>
 * ２．更新カラム名無しのINSERT文<br>
 * ３．update文<br>
 * <br>
 * 但し、エラーチェックは一切行わないので、必ず正常に実行できるとは限りません。
 * 
 * @author sima
 * 
 */
public class CsvImportSqlExecute {
	private SchemaTableName tbl;

	private ArrayList<String> lowHeader;

	private ArrayList<String> primeKeys;

	private ArrayList<ColumnInfoTable> columnInfoTables;

	private ArrayList<ColumnInfoResult> columnInfoResult;

	/**
	 * 要素のの並びは主キー定義順で、保持する値はCSVファイルヘッダの何番目か？
	 */
	private ArrayList<Integer> primeIdxOnCsv;

	/**
	 * 要素のの並びは主キー定義順で、保持する値はテーブル定義の何番目か？
	 */
	private ArrayList<Integer> primeIdxOnTable;

	/**
	 * 要素のの並びはCSVファイルのヘッダ順で、保持する値はテーブル定義の何番目か？
	 */
	private ArrayList<Integer> columnIdxOnTable;

	private Connection con;

	private String nullValue;

	/**
	 * @param tbl
	 *            テーブル名
	 * @param rowHeader
	 *            csvファイルのカラム見出し
	 * @throws SQLException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws InstantiationException
	 * @throws NoSuchMethodException
	 * @throws ClassNotFoundException
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 * @throws JdbcacsessException 
	 */
	public CsvImportSqlExecute(Connection con, SchemaTableName tbl, Vector<String> rowHeader) throws SQLException,
			SecurityException, IllegalArgumentException, ClassNotFoundException, NoSuchMethodException,
			InstantiationException, IllegalAccessException, InvocationTargetException, JdbcacsessException {
		this.con = con;
		this.tbl = tbl;

		this.lowHeader = new ArrayList<String>();
		// 大文字・小文字をＤＢが要求する方に変換する
		for (String str : rowHeader) {
			this.lowHeader.add(SqlExec.convertStoresIdentifiers(str));
		}

		// インポートするテーブルのカラム名Listを作成する
		this.columnInfoTables = SqlExec.getColumnMames(tbl);
		if (this.columnInfoTables.isEmpty()) {
			throw new SQLException("テーブル定義が見つかりません[" + tbl.toString() + "]");
		}

		// ColumnInfoTableからColumnInfoResultを作成しsqlMappingInfoを反映する
		columnInfoResult = new ArrayList<ColumnInfoResult>();
		SqlMappingInfo smi = new SqlMappingInfo();
		primeKeys = new ArrayList<String>();
		for (ColumnInfoTable cit : columnInfoTables) {
			// columnInfoTables から ColumnInfoResult へ変換
			ColumnInfoResult cir = new ColumnInfoResult(cit);
			// sqlマッピングの情報を ColumnInfoResult に反映する
			smi.search(cir, false);

			columnInfoResult.add(cir);

			// インポートするテーブルの主キーカラム名Listを作成する
			if (cit.isPrimaryKey()) {
				this.primeKeys.add(cit.getColumnName());
			}
		}

		// 主キーカラムのWhere句作成用にCSVカラムインデックスリストを作成
		this.primeIdxOnCsv = new ArrayList<Integer>();
		this.primeIdxOnTable = new ArrayList<Integer>();
		for (String str : primeKeys) {
			int idx;
			idx = rowHeader.indexOf(str);
			if (idx != -1) {
				primeIdxOnCsv.add(idx);
			}
			idx = indexOfcolumnInfoTables(str);
			if (idx != -1) {
				primeIdxOnTable.add(idx);
			}
		}

		// カラムのinsert/update対象カラム編集用にCSVカラムインデックスリストを作成
		this.columnIdxOnTable = new ArrayList<Integer>();
		for (String str : rowHeader) {
			int idx = indexOfcolumnInfoTables(str);
			if (idx != -1) {
				columnIdxOnTable.add(idx);
			}

		}

		nullValue = JFrameSetting.getColumnOutNullValue();
	}

	private int indexOfcolumnInfoTables(String columnName) {
		for (int i = 0; i < columnInfoTables.size(); i++) {
			ColumnInfoTable element = columnInfoTables.get(i);
			if (element.getColumnName().equals(columnName)) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * csvファイルのカラム見出しが、テーブル定義カラムの全てを、含んでいるか調べる
	 * 
	 * @return 全て含んでいる時：空文字列<br>
	 *         そうでない時：エラーがあったカラムのメッセージ<br>
	 */
	public String hasAllColumn() {
		// CSVファイルカラム名のチェック
		for (String str : lowHeader) {
			if (indexOfcolumnInfoTables(str) == -1) {
				// "CSVの見出しカラムがテーブル定義に無い\n"
				return "カラム名:[" + str + "] テーブル名:[" + tbl + "]";
			}
		}
		return "";
	}

	/**
	 * csvファイルのカラム見出しが、テーブル定義の主キーの全てを、含んでいるか調べる
	 * 
	 * @return 全て含んでいる時：空文字列<br>
	 *         そうでない時：エラーがあったカラムのメッセージ<br>
	 */
	public String hasPrimeKey() {
		if (primeKeys.size() == 0) {
			return "主キーが無いテーブルはupdate指定出来ません";
		}
		for (String str : primeKeys) {
			if (lowHeader.indexOf(str) == -1) {
				// "主キー定義カラムがCSVの見出しに無い\n"
				return "カラム名:[" + str + "] テーブル名:[" + tbl + "]";
			}
		}
		return "";
	}

	/**
	 * CSVファイルのデータリストから更新カラム名無しのINSERT文を作成し実行します。<br>
	 * リストは、その順番にvalue句を組み立てるだけなので、 テーブル定義のカラム順序を厳密に守る必要があります。
	 * 
	 * @param items
	 *            csvファイルのデータ
	 * @return 処理件数
	 */
	private PrepareExecuteStatement insertOvpCmd = null;

	public int execInsertOnlyValuePhrase(Vector<String> items) throws SQLException, SecurityException,
			IllegalArgumentException, ClassNotFoundException, NoSuchMethodException, InstantiationException,
			IllegalAccessException, InvocationTargetException {
		if (insertOvpCmd == null) {
			insertOvpCmd = new PrepareExecuteStatement();
			insertOvpCmd.addStm("INSERT INTO " + tbl.getCompleteTableName());
			insertOvpCmd.addStm(editValuesPhrase(items.size()));
			insertOvpCmd.prepare(con);
		}
		for (int columnInfoTableIdx = 0; columnInfoTableIdx < items.size(); columnInfoTableIdx++) {
			addItemValue(insertOvpCmd, items.get(columnInfoTableIdx), columnInfoTableIdx);
		}
		return insertOvpCmd.executeUpdate();
	}

	private String editValuesPhrase(int size) {
		PrefixStringBuffer psb = new PrefixStringBuffer(" VALUES(", ",");
		for (int i = 0; i < size; i++) {
			psb.appendItem("?");
		}
		return psb.append(")").getString();
	}

	/**
	 * CSVファイルのデータリストからINSERT文を作成し実行します。<br>
	 * カラム名を指定したINSERT文が作成されるので、 リストはテーブル定義のカラム順と異なっていても問題ありません。<br>
	 * 但し、コンストラクタで指定したカラム名の順序と一致する必要があります。
	 * 
	 * @param items
	 *            csvファイルのデータ
	 * @return 処理件数
	 */
	private PrepareExecuteStatement insertCavCmd = null;

	public int execInsertColumnAndValue(Vector<String> items) throws SQLException, SecurityException,
			IllegalArgumentException, ClassNotFoundException, NoSuchMethodException, InstantiationException,
			IllegalAccessException, InvocationTargetException {
		if (columnIdxOnTable.size() != items.size()) {
			throw new SQLException("見出し項目数:" + columnIdxOnTable.size() + "　と明細項目数:" + items.size() + "　が一致しません");
		}
		if (insertCavCmd == null) {
			insertCavCmd = new PrepareExecuteStatement();
			insertCavCmd.addStm("INSERT INTO " + tbl.getCompleteTableName());

			PrefixStringBuffer psb = new PrefixStringBuffer(" (",",");
			for (Integer idx : columnIdxOnTable) {
				psb.appendItem(columnInfoTables.get(idx).getColumnName());
			}
			insertCavCmd.addStm(psb.append(")").getString());
			
			insertCavCmd.addStm(editValuesPhrase(items.size()));
			insertCavCmd.prepare(con);
		}

		for (int i = 0; i < items.size(); i++) {
			addItemValue(insertCavCmd, items.get(i), columnIdxOnTable.get(i));
		}
		return insertCavCmd.executeUpdate();
	}

	/**
	 * CSVファイルのデータリストからUPDATE文を作成し実行します。<br>
	 * hasAllColumnが空文字列を返却する場合だけ、正常なSQL文を生成できます。<br>
	 * また、更新を行う為には主キーが必要なので、hasPrimeKeyが 空文字列を返却する事も必要です。
	 * 
	 * @param items
	 *            csvファイルのデータ
	 * @return 処理件数
	 */
	private PrepareExecuteStatement updateCmd = null;

	public int execUpdate(Vector<String> items) throws SQLException, SecurityException, IllegalArgumentException,
			ClassNotFoundException, NoSuchMethodException, InstantiationException, IllegalAccessException,
			InvocationTargetException {
		if (columnIdxOnTable.size() != items.size()) {
			throw new SQLException("見出し項目数:" + columnIdxOnTable.size() + "　と明細項目数:" + items.size() + "　が一致しません");
		}
		if (updateCmd == null) {
			updateCmd = new PrepareExecuteStatement();
			updateCmd.addStm("UPDATE " + tbl.getCompleteTableName());
			PrefixStringBuffer psb;

			psb = new PrefixStringBuffer(" SET "," ,");
			for (int i = 0; i < items.size(); i++) {
				psb.appendItem(columnInfoTables.get(columnIdxOnTable.get(i)) + "=?");
			}
			updateCmd.addStm(psb.getString());
			
			psb = new PrefixStringBuffer(" WHERE "," AND ");
			for (Integer idx : primeIdxOnCsv) {
				psb.appendItem(lowHeader.get(idx) + "=?");
			}
			updateCmd.addStm(psb.getString());
			
			updateCmd.prepare(con);
		}

		// 更新する値を編集する
		for (int i = 0; i < items.size(); i++) {
			addItemValue(updateCmd, items.get(i), columnIdxOnTable.get(i));
		}

		// 条件の値を編集する
		for (int i = 0; i < primeIdxOnCsv.size(); i++) {
			int primeIdx_csv = primeIdxOnCsv.get(i);
			int primeIdx_table = primeIdxOnTable.get(i);
			addItemValue(updateCmd, items.get(primeIdx_csv), primeIdx_table);
		}
		return updateCmd.executeUpdate();
	}

	private PrepareExecuteStatement selectCountCmd = null;

	public int querySelectCount(Vector<String> items) throws SQLException, SecurityException, IllegalArgumentException,
			ClassNotFoundException, NoSuchMethodException, InstantiationException, IllegalAccessException,
			InvocationTargetException {
		if (selectCountCmd == null) {
			selectCountCmd = new PrepareExecuteStatement();
			selectCountCmd.addStm("SELECT COUNT(*) FROM " + tbl.getCompleteTableName());
			PrefixStringBuffer psb = new PrefixStringBuffer(" WHERE ", " AND ");
			for (Integer idx : primeIdxOnCsv) {
				psb.appendItem(lowHeader.get(idx) + "=?");
			}
			selectCountCmd.addStm(psb.getString());
			selectCountCmd.prepare(con);
		}
		// 条件の値を編集する
		for (int i = 0; i < primeIdxOnCsv.size(); i++) {
			int primeIdx_csv = primeIdxOnCsv.get(i);
			int primeIdx_table = primeIdxOnTable.get(i);
			addItemValue(selectCountCmd, items.get(primeIdx_csv), primeIdx_table);
		}
		ResultSet rs = selectCountCmd.executeQuery();
		rs.next();
		return rs.getInt(1);
	}

	private void addItemValue(PrepareExecute cmd, String item, int columnInfoTableIdx) throws IllegalArgumentException,
			InstantiationException, IllegalAccessException, InvocationTargetException {

		ColumnInfoResult cir = columnInfoResult.get(columnInfoTableIdx);

		if (item.equals(nullValue)) {
			cmd.addParm(null, cir);
			return;
		}

		// String のデータから 指定されたコンストラクタのオブジェクトを生成
		Object value = cir.getConstructor().newInstance(new Object[] { item });

		// パラメータを設定
		cmd.addParm(value, cir);

	}
}
