/**
 * 
 */
package jdbcacsess.gui.cell;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.sql.column.Binary;
import jdbcacsess.sql.column.BinaryInputStream;

/**
 * @author sima
 * 
 */
public class JTableHexViewerModel extends AbstractTableModel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 6789596886192425027L;

	/**
	 * 画面１行のバイト数
	 */
	private static final int ROWBYTE = 32;

	/**
	 * １ブロックの行数
	 */
	private static final int BLOCKCNT = 128;

	/**
	 * 入力となるストリーム
	 */
	private InputStream in;

	/**
	 * 画面表示用のバッファ
	 */
	private ArrayList<byte[]> buf;

	/**
	 * 最後迄データを読み込んだか
	 */
	private boolean endOfStream;

	private int rowCount;

	private JFrameViewerHex jFrameHexViewer;

	/**
	 * コンストラクタ
	 * 
	 * @param inputStream
	 *            ストリーム
	 * @param size
	 *            全体のバイト数。不明な時は-1
	 * @param viewer
	 *            ファイルを最後迄読み込んだ時に通知するオブジェクト
	 */
	public JTableHexViewerModel(InputStream inputStream, long size,
			JFrameViewerHex viewer) {
		super();
		in = inputStream;
		jFrameHexViewer = viewer;
		endOfStream = false;

		// バッファVectorの初期サイズ、増分サイズをデフォルトで1MB分を管理できる大きさ
		int bufSize = (1 * 1024 * 1024) / (ROWBYTE * BLOCKCNT);

		long blockCnt = 0;
		if (size > 0) {
			// サイズが指定された時は、計算する。
			blockCnt = size / (ROWBYTE * BLOCKCNT);
			if (blockCnt < bufSize) {
				// デフォルト分では大きすぎるので、必要分だけにする
				bufSize = (int) blockCnt + 1;
			}
			rowCount =(int)  ( size / ROWBYTE ) + ( size % ROWBYTE > 0 ? 1 : 0);
		}else{
			// サイズが不明の時は行数も不明
			rowCount = -1;
		}

		// バッファを作成
		buf = new ArrayList<byte[]>(bufSize);
		System.out.println("size(byte)=" + size + " blockCnt=" + blockCnt
				+ " bufSize=" + bufSize + " rowCount=" + rowCount);

		// 読み込みを行う。
		try {
			read(1);
		} catch (IOException e) {
			clear();
			JDialogMessage.errorDialog(e);
		} catch (OutOfMemoryError error) {
			clear();
			JDialogMessage.errorDialog(new Exception(error));
		}
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		if (rowCount>-1) {
			return rowCount;
		}else{
			if (endOfStream){
				rowCount = getRealRowCount() ;
				return rowCount;
			}
		}
		// 最後迄読み込んでないので、とりあえず現時点のbufの大きさから求める
		return (buf.size() * ROWBYTE * BLOCKCNT);
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		// １行のバイト数に相対位置列を加算し列数とする
		return ROWBYTE + 1;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int column) {
		if (column == 0) {
			return "相対位置";
		}
		return String.format("%02X", column - 1);
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		// エラー発生後はNULLになってる
		if (in == null) {
			return null;
		}
		// 最左端は相対位置
		if (columnIndex == 0) {
			return String.format("%04X", rowIndex * ROWBYTE);
		}
		// 画面表示インデックスを１左へずらし、バッファの添え字と同じにする。
		columnIndex = columnIndex - 1;

		// 画面上の行インデックスに対応する、ブロックインデックスを算出
		int blockIndex = rowIndex / BLOCKCNT;

		try {
			// ブロックインデックス迄の読み取り保証を依頼
			read(blockIndex);
		} catch (IOException e) {
			clear();
			JDialogMessage.errorDialog(e);
			return null;
		} catch (OutOfMemoryError error) {
			clear();
			JDialogMessage.errorDialog(new Exception(error));
			return null;
		}

		// 読み取り保証できなかったので、バッファに存在しない。
		// getRowCount が仮想位置を示すので、rowIndexバッファい以上の値を受取る
		if (buf.size() <= blockIndex) {
			return null;
		}

		// バッファの１要素 = 1ブロック を取り出す
		byte[] blockData = buf.get(blockIndex);

		// ブロック内のバイト位置を算出
		int idx = (rowIndex % BLOCKCNT) * ROWBYTE + columnIndex;

		// １バイト分を取り出す
		if (idx < blockData.length) {
			return Binary.byteToHexString(blockData[idx]);
		}

		// １行の半端なデータをnullで返す
		return null;
	}

	/**
	 * ブロックインデックス迄読み込む
	 * 
	 * @param blockIndex
	 * @throws IOException
	 */
	private void read(int blockIndex) throws IOException {
		if (endOfStream) {
			return;
		}
		if (blockIndex < buf.size()) {
			return;
		}

		// 指示行まで読み込む
		byte[] rowBuf;
		int lastIdx = getRealRowCount();
		while ((rowBuf = BinaryInputStream.readBytes(in, ROWBYTE * BLOCKCNT)) != null) {
			buf.add(rowBuf);
			if (blockIndex < buf.size()) {
				// 指定分読み込んだので抜ける
				break;
			}
		}

		// 終端まで読み込んだ時
		if (rowBuf == null) {
			endOfStream = true;
			in.close();
			jFrameHexViewer.setDataSize(getDataSize());
			System.out.println("読み込みeof" + " buf.size()=" + buf.size());
		} else {
			System.out.println("読み込みbuf完了" + " buf.size()=" + buf.size());
		}

		// 追加した行を通知する。Swingは描画と新たなrowCountの取り込みが行われる？
		fireTableRowsInserted(lastIdx, getRealRowCount());

		return;
	}

	/**
	 * バッファ状態に基づいた画面表示の行数を返却
	 * 
	 * @return 実際の行数
	 */
	private int getRealRowCount() {
		// ０バイトファイルの場合
		if (buf.size() == 0) {
			return 0;
		}

		// 最後のブロックを取得
		byte[] block = buf.get(buf.size() - 1);
		// 最終ブロック内の行数を計算(0バイトのブロックは存在しない)
		int rowCntInLastBlock = ((block.length - 1) / ROWBYTE) + 1;
		// 有効行数を返却
		return (buf.size() - 1) * BLOCKCNT + rowCntInLastBlock;
	}

	/**
	 * バッファ状態に基づいた読み込みバイト数を返却
	 * 
	 * @return 読み込み済バイト数
	 */
	private long getDataSize() {
		// ０バイトファイルの場合
		if (buf.size() == 0) {
			return 0;
		}

		// 最後のブロックを取得
		byte[] block = buf.get(buf.size() - 1);

		return (buf.size() - 1) * BLOCKCNT * ROWBYTE + block.length;
	}

	/**
	 * リソースを解放する
	 */
	public void clear() {
		if (in != null) {
			try {
				in.close();
				in = null;
			} catch (IOException dummy) {
			}
		}
		if (buf != null) {
			buf.clear();
			buf.trimToSize();
		}
	}

}
