/**
 * 
 */
package jdbcacsess.gui.common;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.UnsupportedLookAndFeelException;

import jdbcacsess.JdbcacsessException;
/**
 * JFrameの共通機能を提供します。
 * JFrameの閉じる機能、コンポーネントのサイズと位置の保存復元、UIが変更になったときのコンポーネント変更等です。
 * また、JFameが閉じるときにメソッドをオーバライドすれば独自の終了処理を追加できます。
 * 
 * @author sima
 * 
 */
public abstract class JFrameBase extends JFrame {

	private static final long serialVersionUID = 1007026627927273468L;

	/**
	 * 表示されている全てのJFrameを保持します
	 */
	private static List<JFrameBase> jframeBases = Collections.synchronizedList(new ArrayList<JFrameBase>());

	/**
	 * パラメータ設定に使用
	 */
	protected ComponentProperty omponentProperty;
	
	/**
	 * 画面インスタンスの内部リスト登録とクローズ時の削除リスナー設定
	 */
	public JFrameBase(){
		omponentProperty = new ComponentProperty();
		jframeBases.add(this);
		addWindowListener(new java.awt.event.WindowAdapter() {
			@Override
			public void windowClosing(java.awt.event.WindowEvent e) {
				try {
					closeFrame();
				} catch (JdbcacsessException e1) {
					JDialogMessage.errorDialog(e1);
				}
			} 
		});
	}

	/**
	 * SwingのsetVisibleをオーバライドし、画面表示時のサイズ・位置を復元する
	 */
	@Override
	public void setVisible(boolean b){
		if (b){
			if ( getAttrKey()!=null && getAttrValue()!=null ){
				if ( !getAttrKey().equals("") && !getAttrValue().equals("") ){
					omponentProperty.appendCondition(getAttrKey(), getAttrValue());
				}
			}
			
			omponentProperty.restoreWindowPosition(this, getSize().width, getSize().height);
		}
		super.setVisible(b);
	}

	/**
	 * １画面複数表示の時に、画面キーを返却
	 * @return
	 */
	abstract public String getAttrKey();
	/**
	 * １画面複数表示の時に、画面キーの値を返却
	 * @return
	 */
	abstract public String getAttrValue();
	
	/**
	 * 閉じるボタンがある時は、ウィンドウを閉じる時に呼び出してください。
	 * @throws JdbcacsessException 
	 */
	final public void closeFrame() throws JdbcacsessException {
		omponentProperty.storeWindowPosition(this);
		jframeBases.remove(this);
		dispose();
		frameClosing();
	}

	/**
	 * ウィンドウが閉じる時に行う処理をオーバライドして下さい。
	 * @throws JdbcacsessException 
	 */
	abstract public void frameClosing() throws JdbcacsessException ;

	/**
	 * UIが変更になったときは呼び出してください。
	 * @throws UnsupportedLookAndFeelException 
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 * @throws ClassNotFoundException 
	 */
	final public static void changeUI() throws ClassNotFoundException, InstantiationException, IllegalAccessException, UnsupportedLookAndFeelException {
		GUIManager.setUIManager();
		for (JFrameBase item : jframeBases) {
			SwingUtilities.updateComponentTreeUI(item);
		}
	}

	/**
	 * 自分以外の全Frameを閉じます
	 * @throws JdbcacsessException 
	 */
	final public void closeAnotherFrame() throws JdbcacsessException {
		ArrayList<JFrameBase> tmp = new ArrayList<JFrameBase>(jframeBases);
		for (JFrameBase item : tmp) {
			if (item != this) {
				item.closeFrame();
			}
		}
	}
}
