package jdbcacsess.sql;

import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import jdbcacsess.LogOutput;
import jdbcacsess.gui.CellEditorSqlTable;
import jdbcacsess.gui.CellRendererSqlTable;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.sql.column.GetColumnResultSet;


/**
 * １項目分のカラム名情報を保持する
 */

public class ColumnInfoResult extends ColumnInfo {

	/**
	 * 取得結果オブジェクトのクラス名
	 */
	private String columnClassName;
	
	/**
	 * 取得結果オブジェクトのクラス
	 */
	private Class<?> columnClass;

	
	/**
	 * 結果セットからの値を取得する時の独自クラスのオブジェクト(方法１）
	 */
	public GetColumnResultSet getColumnResultSet;
	
	/**
	 * 検索結果セットからの値を取得する時のResultSetクラスのメソッド名(方法２）
	 */
	public Method methodOfResultSet;

	
	/**
	 * カラムデータを表示するレンダラ－(nullの場合はツールデフォルトが使用されます)
	 */
	private TableCellRenderer tableCellRenderer;
	
	/**
	 * カラムデータを編集するエディター(nullの場合はツールデフォルトが使用されます)
	 */
	private TableCellEditor tableCellEditor;
	
	
	private int precision = 0;

	private int scale = 0;

	/**
	 * @param columnIndex
	 *            １から始まるカラムインデックス番号 (ColumnInfoで保持します)
	 * @param columnName
	 *            カラム名 (ColumnInfoで保持します)
	 * @param sqlType
	 *            カラムのSqlType (ColumnInfoで保持します)
	 * @param columnTypeName
	 *            データベース独自のデータ型名 (ColumnInfoで保持します)
	 * @param nullable
	 *            NULLを許すか (ColumnInfoで保持します)
	 * 
	 * @param columnClassName
	 *            カラムデータのクラス型
	 * @param precision
	 *            精度
	 * @param scale
	 *            小数点以下の桁数
	 */
	public ColumnInfoResult(int columnIndex, String columnName, int columnType,
			String columnTypeName, int nullable, String columnClassName,
			int precision, int scale) {

		super(columnIndex, columnName, columnType, columnTypeName, nullable);

		this.columnClassName = columnClassName;
		this.precision = precision;
		this.scale = scale;

		setTableCellRenderer(new CellRendererSqlTable());
		setTableCellEditor(new CellEditorSqlTable());
	}

	/**
	 * ColumnInfoTableからコンストラクタを作る
	 * 
	 * @param cit
	 */
	public ColumnInfoResult(ColumnInfoTable cit) {
		this(cit.getColumnIndex(), cit.getColumnName(), cit.getSqlType().getValue(),
				cit.getColumnTypeName(), 0, java.lang.String.class.getName(), 0, 0);
	}

	private static Map<String, Class<?>> primitiveMap;
	static {
		primitiveMap = new HashMap<String, Class<?>>();
		primitiveMap.put("byte", Byte.class);
		primitiveMap.put("short", Short.class);
		primitiveMap.put("int", Integer.class);
		primitiveMap.put("long", Long.class);
		primitiveMap.put("float", Float.class);
		primitiveMap.put("double", Double.class);
		primitiveMap.put("char", Character.class);
		primitiveMap.put("boolean", Boolean.class);
	}
	
	/**
	 * コンストラクタで指定されたクラス名からクラスをロードして返却
	 * プリミティブ型が設定されていた場合は、ラッパークラスに変換して返却
	 * ただし、　setColumnClassを使用して、書き換えられていたら、その値をそのまま返却
	 * @return
	 */
	public Class<?> getColumnClass() {
		
		if (this.columnClass==null){
			/*
			 * oracleのBINARY_FLOAT,BINARY_DOUBLEは、信じられない事にgetMetaData().getColumnClassName()がnull。
			 * しょうがないので、nullの時は、java.lang.Objectで処理させる。
			 */
			String name = (columnClassName == null ? "java.lang.Object" : columnClassName);

			this.columnClass = primitiveMap.get(columnClassName);
			if (columnClass==null){
				try {
					this.columnClass = SqlExec.classForName(name);
				} catch (Exception e) {
					JDialogMessage.errorDialog(e);
					this.columnClass = Object.class;
				}
			}
		}
		return columnClass;
	}

	/**
	 * クラスを設定する。
	 * また、コンストラクタで設定したクラス名も、変更する。
	 * @param columnClass
	 */
	public void setColumnClass(Class<?> columnClass) {
		this.columnClassName = columnClass.getName();
		this.columnClass = null;
	}
	
	public void setColumnClassName(String columnClassName) {
		this.columnClassName = columnClassName;
		this.columnClass = null;
	}
	
	public String  getColumnClassName() {
		return this.columnClassName;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.sql.ColumnInfo#getColumnComment()
	 */
	@Override
	public String getColumnComment() {
		String str = super.getColumnComment();
		if (precision <= 0) {
			return str;
		}
		if (scale <= 0) {
			return str + "(" + precision + ")";
		}
		return str + "(" + precision + "." + scale + ")";
	}

	/**
	 * @param tableCellRenderer 設定する tableCellRenderer
	 */
	public void setTableCellRenderer(TableCellRenderer tableCellRenderer) {
		this.tableCellRenderer = tableCellRenderer;
	}

	/**
	 * @return tableCellRenderer
	 */
	public TableCellRenderer getTableCellRenderer() {
		return tableCellRenderer;
	}

	/**
	 * @param tableCellEditor 設定する tableCellEditor
	 */
	public void setTableCellEditor(TableCellEditor tableCellEditor) {
		this.tableCellEditor = tableCellEditor;
	}

	/**
	 * @return tableCellEditor
	 */
	public TableCellEditor getTableCellEditor() {
		return tableCellEditor;
	}

	public int getPrecision() {
		return precision;
	}

	public int getScale() {
		return scale;
	}

	public Constructor<?> getConstructor() {
		Constructor<?> constructor=null;
		try {
			
			constructor = getColumnClass().getConstructor(new Class[] { String.class });

		} catch (Exception e) {
			
			if (! isExistsSqlMapping()){
				JDialogMessage.infoDialog(getColumnClass().getName()+"は、String を引数とするコンストラクタを持っていないので、更新時エラーが起きるかもしれません。\n" +
					"「SQLマッピング設定」で、取得クラスとsql.Types値を設定してみてください。\n\n" +
					"例えば、取得クラス= toString() , sql.Types値= Types.VARCHAR 等です。" ,"強制的にStringとして更新");
			}
			try {
				constructor = String.class.getConstructor(new Class[] { String.class });
			} catch (Exception e1) {
				e1.printStackTrace();
			}
		}
		LogOutput.fine("constructor:" + constructor);
		return constructor;
	}
	
	private boolean existsSqlMapping = false;

	public boolean isExistsSqlMapping() {
		return existsSqlMapping;
	}

	public void setExistsSqlMapping(boolean existsSqlMapping) {
		this.existsSqlMapping = existsSqlMapping;
	}
	

}