package jdbcacsess.sql;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;

import jdbcacsess.gui.JFrameMain;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.sql.column.Binary;
import jdbcacsess.sql.column.BinaryInputStream;
import jdbcacsess.sql.column.CharacterInputStream;

public abstract class PrepareExecute {

	protected ArrayList<PrepareExecuteParm> parmList;

	protected PreparedStatement prepared;

	protected StringBuffer statement;

	static class PrepareExecuteParm {
		int idx;

		Object x;

		ColumnInfo columnInfo;
		
		public PrepareExecuteParm(Object x, ColumnInfo columnInfo) {
			idx = 0;
			this.x = x;
			this.columnInfo = columnInfo;
		}

		public PrepareExecuteParm(int idx, Object x, ColumnInfo columnInfo) {
			this.idx = idx;
			this.x = x;
			this.columnInfo = columnInfo;
		}

	}

	public PrepareExecute() {
		statement = new StringBuffer();
		parmList = new ArrayList<PrepareExecuteParm>();
		prepared = null;
	}

	/**
	 * SQL文を連結します
	 * 
	 * @param stm
	 */
	public void addStm(String stm) {
		statement.append(stm);
	}

	/**
	 * ＳＱＬ文を返却します。パラメータの値は含まれません。
	 * 
	 * @return
	 */
	public String getStm() {
		return statement.toString();
	}

	/**
	 * パラメータを内部リストに追加します
	 * 
	 * @param value
	 */
	public void addParm(Object value, ColumnInfo columnInfo) {
		this.parmList.add(new PrepareExecuteParm(value, columnInfo));
	}

	/**
	 * パラメータを内部リストに追加します
	 * 
	 * @param value
	 */
	public void addParm(int idx, Object value, ColumnInfo columnInfo) {
		this.parmList.add(new PrepareExecuteParm(idx, value, columnInfo));
	}

	abstract public void prepare(Connection con) throws SQLException;

	/**
	 * 内部リストで保持しているパラメータをPreparedStatementのパラメータに設定します
	 * 
	 * @throws SQLException
	 */
	final void setParm() throws SQLException {
		for (int i = 0; i < parmList.size(); i++) {
			PrepareExecuteParm pep = parmList.get(i);

			Object value = pep.x;
			ConstSqlTypes sqlMappingSqlType = pep.columnInfo.getSqlMappingSqlType();
			
			// インデックスがゼロの時は順番に自動生成
			int idx = (parmList.get(i).idx == 0 ? i + 1 : pep.idx);

			if (value == null) {
				if (sqlMappingSqlType == ConstSqlTypes.UNUSE) {
					//setNullは必ずsqltypesを指定する
					msg("setNull("+idx+","+pep.columnInfo.getSqlTypeInt()+")");
					prepared.setNull(idx, pep.columnInfo.getSqlTypeInt());
				}else{
					msg("setNull("+idx+","+sqlMappingSqlType+")");
					prepared.setNull(idx, sqlMappingSqlType.getValue());
				}
			} else {
				if (value instanceof BinaryInputStream) {
					BinaryInputStream bis = (BinaryInputStream) value;
					msg("setBinaryStream("+","+idx+","+bis.getInputStream().getClass()+","+bis.getSize()+")");
					prepared.setBinaryStream(idx, bis.getInputStream(), bis.getSize());

				} else if (value instanceof CharacterInputStream) {
					CharacterInputStream cis = (CharacterInputStream) value;
					msg("setCharacterStream("+","+idx+","+cis.getReader().getClass()+","+cis.getSize()+")");
					prepared.setCharacterStream(idx, cis.getReader(), cis.getSize());

				} else if (value instanceof Binary) {
					msg("setBytes("+","+idx+","+((Binary) value).getValue().getClass()+")");
					prepared.setBytes(idx, ((Binary) value).getValue());

				} else {
					if (sqlMappingSqlType == ConstSqlTypes.UNUSE) {
						//setObjectはsqltypesを省略
						msg("setObject("+","+idx+","+value.getClass()+")");
						prepared.setObject(idx, value);
					} else {
						msg("setObject("+","+idx+","+value.getClass()+","+sqlMappingSqlType+")");
						prepared.setObject(idx, value, sqlMappingSqlType.getValue());
					}

				}
			}
		}

		if (JFrameMain.debugMode) {
			StringBuilder sb = new StringBuilder("PrepareExecute:" + statement + " [");
			for (PrepareExecuteParm o : parmList) {
				sb.append((o.x == null ? "null" : o.x.toString() ));
				sb.append(",");
			}
			sb.append("]\n");
			JFrameMain.getInstanceJPanelPrepareStatementLog().appendLog(sb);
		}
	}

	/**
	 * パラメータを保持している内部リストをクリアします
	 * 
	 * @throws SQLException
	 */
	final void clearParm() throws SQLException {
		for (Object item : parmList) {
			if (item instanceof BinaryInputStream) {
				BinaryInputStream bis = (BinaryInputStream) item;
				try {
					bis.getInputStream().close();
				} catch (IOException e) {
					JDialogMessage.errorDialog(e);
				}
			}
			if (item instanceof CharacterInputStream) {
				CharacterInputStream cis = (CharacterInputStream) item;
				try {
					cis.getReader().close();
				} catch (IOException e) {
					JDialogMessage.errorDialog(e);
				}
			}
		}
		parmList.clear();
	}

	public void close() throws SQLException {
		if (prepared != null) {
			prepared.close();
		}
	}

	public boolean isHavingInputParameter() {
		ParameterMetaData pmd;
		try {
			pmd = prepared.getParameterMetaData();
			if (pmd == null) {
				return false;
			}
			for (int i = 1; i < pmd.getParameterCount() + 1; i++) {
				if (pmd.getParameterMode(i) == ParameterMetaData.parameterModeIn
						|| pmd.getParameterMode(i) == ParameterMetaData.parameterModeInOut) {
					return true;
				}
			}
		} catch (Error e) {
			e.printStackTrace();
			return false;
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return false;
	}

	public PreparedStatement getPrepared() {
		return prepared;
	}

	void msg(String m){
		if (JFrameMain.debugMode) {
			JFrameMain.getInstanceJPanelPrepareStatementLog().appendLog(m +"\n");
		}
	}
}
