package jdbcacsess.sql;

/**
 * ＳＱＬ指定形式の完全テーブル名の生成を管理します
 * 
 * @author sima
 * 
 */

public class SchemaTableName implements Comparable<SchemaTableName> {
	private String schemaName;

	private String tableName;

	private String aliasName;
	/**
	 * スキーマ名とテーブル名を空文字列として生成します
	 */
	public SchemaTableName() {
		this.schemaName = "";
		this.tableName = "";
		this.aliasName = "";
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}

		if (obj == null) {
			return false;
		}

		if (this.getClass() != obj.getClass()) {
			return false;
		}

		SchemaTableName other = (SchemaTableName) obj;

		// 何故か SQLiteでは、getTables と getColumns の大文字小文字が異なる。たぶんgetTablesの実装が間違ってる。
		// しょうがないので、比較は大文字に統一して行う。保持するデータはドライバの値のままとする。
		// なんかヤバげな対応！
		return other.getSchemaName().toUpperCase().equals(this.schemaName.toUpperCase())
				&& other.getTableName().toUpperCase().equals(this.tableName.toUpperCase());
	}

	@Override
	public int hashCode() {
		return (this.schemaName + this.tableName).hashCode();
	}

	/**
	 * スキーマ名．テーブル名のＳＱＬ指定形式の文字列から、 ピリオドを区切り文字としてスキーマ名とテーブル名に分割して生成します。
	 * 
	 * @param completeTableName
	 *            ＳＱＬ形式の完全テーブル名
	 */
	public SchemaTableName(String completeTableName) {
		if (completeTableName == null) {
			schemaName = "";
			tableName = "";
			return;
		}
		completeTableName = completeTableName.trim();
		int idx = completeTableName.indexOf('.');
		if (idx == -1) {
			schemaName = "";
			tableName = completeTableName;
			conv();
			return;
		}

		schemaName = completeTableName.substring(0, idx);
		tableName = completeTableName.substring(idx + 1);
		this.aliasName = "";
		conv();
	}

	/**
	 * 既にスキーマ名とテーブル名が分割してある場合の生成手段です
	 * 
	 * @param schemaName
	 *            スキーマ名
	 * @param tableName
	 *            テーブル名
	 */
	public SchemaTableName(String schemaName, String tableName) {
		if (schemaName == null) {
			this.schemaName = "";
		} else {
			this.schemaName = schemaName.trim();
		}
		if (tableName == null) {
			this.tableName = "";
		} else {
			this.tableName = tableName.trim();
		}
		this.aliasName = "";
		conv();
	}

	private void conv() {
		tableName = SqlExec.convertStoresIdentifiers(tableName);
		schemaName = SqlExec.convertStoresIdentifiers(schemaName);
	}

	/**
	 * ＳＱＬ指定形式の完全テーブル名を、スキーマ名．テーブル名として返却します。 スキーマ名が空で生成された場合は、ピリオドは付加されません
	 * 
	 * @return ＳＱＬ形式の完全テーブル名
	 */
	public String getCompleteTableName() {
		if (schemaName.equals("")) {
			return tableName;
		}
		return schemaName + "." + tableName;

	}

	/**
	 * スキーマ名を返却します
	 * 
	 * @return スキーマ名
	 */
	public String getSchemaName() {
		return schemaName;
	}

	/**
	 * テーブル名を返却します
	 * 
	 * @return テーブル名
	 */
	public String getTableName() {
		return tableName;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return getCompleteTableName();
	}

	public int compareTo(SchemaTableName o) {
		return getCompleteTableName().compareTo(o.getCompleteTableName());
	}

	public String getAliasName() {
		return aliasName;
	}

	public void setAliasName(String aliasName) {
		this.aliasName = aliasName;
	}
}
